/*
 * CTcpSocket.h
 *
 *  Created on: Feb 23, 2011
 *      Author: Joel Vidal
 */

//#include "IIODevice.h"

#ifndef CTCPSOCKET_H_
#define CTCPSOCKET_H_

#include <vector>
#include <string>
#include <iostream>
#include <boost/asio.hpp>
#include <boost/thread.hpp>
#include <boost/lexical_cast.hpp>

#include "../IIODevice.h"


class CTcpSocket : public IIODevice { //: private boost::noncopyable??
public:

	typedef std::vector< unsigned char > DataBuffer ;

	CTcpSocket() ;

	~CTcpSocket() ;


	void
	connect( const std::string& IP,
			 const unsigned short port ) ;


	/**
	 * Read a single byte from the I/O device. If no data is
	 * available in the specified number of milliseconds (msTimeout),
	 * then this method will throw ReadTimeout exception. If msTimeout
	 * is 0, then this method will block till data is available.
	 */

	unsigned char
	readByte ( const unsigned int msTimeout = 0 )
		throw ( IIODevice::ReadTimeout,
				std::runtime_error ) ;

	/**
	 * Read the specified number of bytes from the I/O stream. The
	 * method will timeout if no data is received in the specified
	 * number of milliseconds (msTimeout). If msTimeout is 0, then
	 * this method will block till all requested bytes are
	 * received. If numOfBytes is zero, then this method will keep
	 * reading data till no more data is available at the serial
	 * port. In all cases, all read data is available in dataBuffer on
	 * return from this method.
	 */

	void
	read ( DataBuffer& dataBuffer,
		   const unsigned int numOfBytes = 0,
		   const unsigned int msTimeout = 0 )
		throw ( IIODevice::ReadTimeout,
				std::runtime_error ) ;

	/**
	 * Read the until the specified delimiting character is found.
	 * The delimiting character is also stored in dataBuffer.
	 */

	void
	readUntil( DataBuffer& dataBuffer,
			   const unsigned char delimiter = '\n',
			   const unsigned int msTimeout = 0 )
		throw ( IIODevice::ReadTimeout,
				std::runtime_error );

	/**
	 * Read a line of characters from the I/O device.
	 */

	std::string
	readLine ( const unsigned int msTimeout = 0,
			   const unsigned char lineTerminator = '\n' )
		throw ( IIODevice::ReadTimeout,
				std::runtime_error ) ;

	/**
	 * Send a single byte to the I/O device.
	 */

	void
	writeByte ( const unsigned char dataByte )
		throw ( std::runtime_error );

	/**
	 * Write the data from the specified DataBuffer to the I/O device.
	 */

	void
	write ( const DataBuffer& dataBuffer )
		throw ( std::runtime_error );

    /**
     * Write a string to the I/O device.
     */

	void
	write ( const std::string& dataString )
		throw ( std::runtime_error );

	/**
	 * Equivalent to readByte(0).
	 */

	IIODevice&
	operator>>( unsigned char& byte )
		throw ( IIODevice::ReadTimeout,
				std::runtime_error );

	/**
	 * Equivalent to read(dataBuffer, 0, 0).
	 */

	IIODevice&
	operator>>( DataBuffer& dataBuffer )
		throw ( IIODevice::ReadTimeout,
				std::runtime_error ) ;

	/**
	 * Equivalent to readLine(line, 0, '\n').
	 */

	IIODevice&
	operator>>( std::string& line )
		throw ( IIODevice::ReadTimeout,
				std::runtime_error ) ;

	/**
	 * Equivalent to writeByte(dataByte).
	 */

	IIODevice&
	operator<<( const unsigned char dataByte )
		throw ( std::runtime_error ) ;

	/**
	 * Equivalent to write(dataBuffer).
	 */

	IIODevice&
	operator<<( const DataBuffer& dataBuffer )
		throw ( std::runtime_error ) ;

	/**
	 * Equivalent to write(line).
	 */

	IIODevice&
	operator<<( const std::string& line )
		throw ( std::runtime_error ) ;

	void
	close() ;



private:

	boost::asio::io_service io_service_;
	boost::asio::ip::tcp::socket socket_;

	void
	timedReadHandler( boost::optional< boost::system::error_code >* a,
					  const boost::system::error_code& b ) ;
};

#endif /* CTCPSOCKET_H_ */
